@@ -1,19 +1,26 @@
+import os
 import uuid
 from datetime import datetime
-from typing import Any, Callable, Dict, List, Literal, Union
+from typing import Any, Callable, Dict, List, Literal, Optional, Union
 
-from loguru import logger
 from pydantic import BaseModel, Field
 
+from swarms.prompts.ag_prompt import aggregator_system_prompt
 from swarms.structs.agent import Agent
 from swarms.structs.concurrent_workflow import ConcurrentWorkflow
+from swarms.structs.csv_to_agent import AgentLoader
+from swarms.structs.groupchat import GroupChat
+from swarms.structs.hiearchical_swarm import HierarchicalSwarm
+from swarms.structs.majority_voting import MajorityVoting
 from swarms.structs.mixture_of_agents import MixtureOfAgents
+from swarms.structs.multi_agent_orchestrator import MultiAgentRouter
 from swarms.structs.rearrange import AgentRearrange
 from swarms.structs.sequential_workflow import SequentialWorkflow
 from swarms.structs.spreadsheet_swarm import SpreadSheetSwarm
-from tenacity import retry, stop_after_attempt, wait_fixed
 from swarms.structs.swarm_matcher import swarm_matcher
-from swarms.prompts.ag_prompt import aggregator_system_prompt
+from swarms.utils.loguru_logger import initialize_logger
+
+logger = initialize_logger(log_folder="swarm_router")
 
 SwarmType = Literal[
     "AgentRearrange",
@@ -21,52 +28,108 @@ SwarmType = Literal[
     "SpreadSheetSwarm",
     "SequentialWorkflow",
     "ConcurrentWorkflow",
+    "GroupChat",
+    "MultiAgentRouter",
+    "AutoSwarmBuilder",
+    "HiearchicalSwarm",
     "auto",
+    "MajorityVoting",
 ]
 
 
+class Document(BaseModel):
+    file_path: str
+    data: str
+
+
 class SwarmLog(BaseModel):
     """
     A Pydantic model to capture log entries.
     """
 
-    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
-    timestamp: datetime = Field(default_factory=datetime.utcnow)
-    level: str
-    message: str
-    swarm_type: SwarmType
-    task: str = ""
-    metadata: Dict[str, Any] = Field(default_factory=dict)
+    id: Optional[str] = Field(
+        default_factory=lambda: str(uuid.uuid4())
+    )
+    timestamp: Optional[datetime] = Field(
+        default_factory=datetime.utcnow
+    )
+    level: Optional[str] = None
+    message: Optional[str] = None
+    swarm_type: Optional[SwarmType] = None
+    task: Optional[str] = ""
+    metadata: Optional[Dict[str, Any]] = Field(default_factory=dict)
+    documents: List[Document] = []
 
 
 class SwarmRouter:
     """
-    A class to dynamically route tasks to different swarm types based on user selection or automatic matching.
+    A class that dynamically routes tasks to different swarm types based on user selection or automatic matching.
 
-    This class enables users to specify a swarm type or let the system automatically determine the best swarm type for a given task. It then runs the task on the selected or matched swarm type, ensuring type validation, logging, and metadata capture.
+    The SwarmRouter enables flexible task execution by either using a specified swarm type or automatically determining
+    the most suitable swarm type for a given task. It handles task execution while managing logging, type validation,
+    and metadata capture.
+
+    Args:
+        name (str, optional): Name identifier for the SwarmRouter instance. Defaults to "swarm-router".
+        description (str, optional): Description of the SwarmRouter's purpose. Defaults to "Routes your task to the desired swarm".
+        max_loops (int, optional): Maximum number of execution loops. Defaults to 1.
+        agents (List[Union[Agent, Callable]], optional): List of Agent objects or callables to use. Defaults to empty list.
+        swarm_type (SwarmType, optional): Type of swarm to use. Defaults to "SequentialWorkflow".
+        autosave (bool, optional): Whether to enable autosaving. Defaults to False.
+        flow (str, optional): Flow configuration string. Defaults to None.
+        return_json (bool, optional): Whether to return results as JSON. Defaults to False.
+        auto_generate_prompts (bool, optional): Whether to auto-generate agent prompts. Defaults to False.
+        shared_memory_system (Any, optional): Shared memory system for agents. Defaults to None.
+        rules (str, optional): Rules to inject into every agent. Defaults to None.
+        documents (List[str], optional): List of document file paths to use. Defaults to empty list.
+        output_type (str, optional): Output format type. Defaults to "string".
 
     Attributes:
-        name (str): The name of the SwarmRouter instance.
-        description (str): A description of the SwarmRouter instance.
-        max_loops (int): The maximum number of loops to perform.
-        agents (List[Union[Agent, Callable]]): A list of Agent objects to be used in the swarm.
-        swarm_type (SwarmType): The type of swarm to be used, which can be specified or automatically determined.
-        autosave (bool): A flag to enable/disable autosave.
-        flow (str): The flow of the swarm.
-        return_json (bool): A flag to enable/disable returning the result in JSON format.
-        auto_generate_prompts (bool): A flag to enable/disable auto generation of prompts.
-        swarm (Union[AgentRearrange, MixtureOfAgents, SpreadSheetSwarm, SequentialWorkflow, ConcurrentWorkflow]):
-            The instantiated swarm object.
-        logs (List[SwarmLog]): A list of log entries captured during operations.
-        auto_generate_prompt (bool): A flag to enable/disable auto generation of prompts.
+        name (str): Name identifier for the SwarmRouter instance
+        description (str): Description of the SwarmRouter's purpose
+        max_loops (int): Maximum number of execution loops
+        agents (List[Union[Agent, Callable]]): List of Agent objects or callables
+        swarm_type (SwarmType): Type of swarm being used
+        autosave (bool): Whether autosaving is enabled
+        flow (str): Flow configuration string
+        return_json (bool): Whether results are returned as JSON
+        auto_generate_prompts (bool): Whether prompt auto-generation is enabled
+        shared_memory_system (Any): Shared memory system for agents
+        rules (str): Rules injected into every agent
+        documents (List[str]): List of document file paths
+        output_type (str): Output format type
+        logs (List[SwarmLog]): List of execution logs
+        swarm: The instantiated swarm object
 
     Available Swarm Types:
-        - AgentRearrange: Rearranges agents for optimal task execution.
-        - MixtureOfAgents: Combines different types of agents for diverse task handling.
-        - SpreadSheetSwarm: Utilizes spreadsheet-like operations for task management.
-        - SequentialWorkflow: Executes tasks in a sequential manner.
-        - ConcurrentWorkflow: Executes tasks concurrently for parallel processing.
-        - "auto" will automatically conduct embedding search to find the best swarm for your task
+        - AgentRearrange: Optimizes agent arrangement for task execution
+        - MixtureOfAgents: Combines multiple agent types for diverse tasks
+        - SpreadSheetSwarm: Uses spreadsheet-like operations for task management
+        - SequentialWorkflow: Executes tasks sequentially
+        - ConcurrentWorkflow: Executes tasks in parallel
+        - "auto": Automatically selects best swarm type via embedding search
+
+    Methods:
+        run(task: str, device: str = "cpu", all_cores: bool = False, all_gpus: bool = False, *args, **kwargs) -> Any:
+            Executes a task using the configured swarm
+
+        batch_run(tasks: List[str], *args, **kwargs) -> List[Any]:
+            Executes multiple tasks in sequence
+
+        threaded_run(task: str, *args, **kwargs) -> Any:
+            Executes a task in a separate thread
+
+        async_run(task: str, *args, **kwargs) -> Any:
+            Executes a task asynchronously
+
+        concurrent_run(task: str, *args, **kwargs) -> Any:
+            Executes a task using concurrent execution
+
+        concurrent_batch_run(tasks: List[str], *args, **kwargs) -> List[Any]:
+            Executes multiple tasks concurrently
+
+        get_logs() -> List[SwarmLog]:
+            Retrieves execution logs
     """
 
     def __init__(
@@ -77,9 +140,18 @@ class SwarmRouter:
         agents: List[Union[Agent, Callable]] = [],
         swarm_type: SwarmType = "SequentialWorkflow",  # "SpreadSheetSwarm" # "auto"
         autosave: bool = False,
-        flow: str = None,
-        return_json: bool = True,
+        rearrange_flow: str = None,
+        return_json: bool = False,
         auto_generate_prompts: bool = False,
+        shared_memory_system: Any = None,
+        rules: str = None,
+        documents: List[str] = [],  # A list of docs file paths
+        output_type: str = "all",
+        no_cluster_ops: bool = False,
+        speaker_fn: callable = None,
+        load_agents_from_csv: bool = False,
+        csv_file_path: str = None,
+        return_entire_history: bool = True,
         *args,
         **kwargs,
     ):
@@ -89,10 +161,24 @@ class SwarmRouter:
         self.agents = agents
         self.swarm_type = swarm_type
         self.autosave = autosave
-        self.flow = flow
+        self.rearrange_flow = rearrange_flow
         self.return_json = return_json
         self.auto_generate_prompts = auto_generate_prompts
+        self.shared_memory_system = shared_memory_system
+        self.rules = rules
+        self.documents = documents
+        self.output_type = output_type
+        self.no_cluster_ops = no_cluster_ops
+        self.speaker_fn = speaker_fn
         self.logs = []
+        self.load_agents_from_csv = load_agents_from_csv
+        self.csv_file_path = csv_file_path
+        self.return_entire_history = return_entire_history
+
+        if self.load_agents_from_csv:
+            self.agents = AgentLoader(
+                csv_path=self.csv_file_path
+            ).load_agents()
 
         self.reliability_check()
 
@@ -101,8 +187,33 @@ class SwarmRouter:
             f"SwarmRouter initialized with swarm type: {swarm_type}",
         )
 
+        # Handle Automated Prompt Engineering
         self.activate_ape()
 
+        # Handle shared memory
+        if self.shared_memory_system is not None:
+            self.activate_shared_memory()
+
+        # Handle rules
+        if self.rules is not None:
+            self.handle_rules()
+
+    def activate_shared_memory(self):
+        logger.info("Activating shared memory with all agents ")
+
+        for agent in self.agents:
+            agent.long_term_memory = self.shared_memory_system
+
+        logger.info("All agents now have the same memory system")
+
+    def handle_rules(self):
+        logger.info("Injecting rules to every agent!")
+
+        for agent in self.agents:
+            agent.system_prompt += f"### Swarm Rules ### {self.rules}"
+
+        logger.info("Finished injecting rules")
+
     def activate_ape(self):
         """Activate automatic prompt engineering for agents that support it"""
         try:
@@ -132,9 +243,8 @@ class SwarmRouter:
             self._log("error", error_msg)
             raise RuntimeError(error_msg) from e
 
-    @retry(stop=stop_after_attempt(3), wait=wait_fixed(1))
     def reliability_check(self):
-        logger.info("Logger initializing checks")
+        logger.info("Initializing reliability checks")
 
         if not self.agents:
             raise ValueError("No agents provided for the swarm.")
@@ -143,7 +253,9 @@ class SwarmRouter:
         if self.max_loops == 0:
             raise ValueError("max_loops cannot be 0.")
 
-        logger.info("Checks completed your swarm is ready.")
+        logger.info(
+            "Reliability checks completed your swarm is ready."
+        )
 
     def _create_swarm(
         self, task: str = None, *args, **kwargs
@@ -153,6 +265,10 @@ class SwarmRouter:
         SpreadSheetSwarm,
         SequentialWorkflow,
         ConcurrentWorkflow,
+        GroupChat,
+        MultiAgentRouter,
+        MajorityVoting,
+        HierarchicalSwarm,
     ]:
         """
         Dynamically create and return the specified swarm type or automatically match the best swarm type for a given task.
@@ -180,8 +296,22 @@ class SwarmRouter:
                 description=self.description,
                 agents=self.agents,
                 max_loops=self.max_loops,
-                flow=self.flow,
+                flow=self.rearrange_flow,
                 return_json=self.return_json,
+                output_type=self.output_type,
+                return_entire_history=self.return_entire_history,
+                *args,
+                **kwargs,
+            )
+
+        elif self.swarm_type == "HiearchicalSwarm":
+            return HierarchicalSwarm(
+                name=self.name,
+                description=self.description,
+                director=self.agents[0],
+                agents=self.agents,
+                max_loops=self.max_loops,
+                return_all_history=self.return_entire_history,
                 *args,
                 **kwargs,
             )
@@ -189,13 +319,42 @@ class SwarmRouter:
             return MixtureOfAgents(
                 name=self.name,
                 description=self.description,
-                reference_agents=self.agents,
+                agents=self.agents,
                 aggregator_system_prompt=aggregator_system_prompt.get_prompt(),
                 aggregator_agent=self.agents[-1],
                 layers=self.max_loops,
                 *args,
                 **kwargs,
             )
+
+        elif self.swarm_type == "MajorityVoting":
+            return MajorityVoting(
+                name=self.name,
+                description=self.description,
+                agents=self.agents,
+                consensus_agent=self.agents[-1],
+                *args,
+                **kwargs,
+            )
+        elif self.swarm_type == "GroupChat":
+            return GroupChat(
+                name=self.name,
+                description=self.description,
+                agents=self.agents,
+                max_loops=self.max_loops,
+                speaker_fn=self.speaker_fn,
+                *args,
+                **kwargs,
+            )
+
+        elif self.swarm_type == "MultiAgentRouter":
+            return MultiAgentRouter(
+                name=self.name,
+                description=self.description,
+                agents=self.agents,
+                shared_memory_system=self.shared_memory_system,
+                output_type=self.output_type,
+            )
         elif self.swarm_type == "SpreadSheetSwarm":
             return SpreadSheetSwarm(
                 name=self.name,
@@ -212,6 +371,10 @@ class SwarmRouter:
                 description=self.description,
                 agents=self.agents,
                 max_loops=self.max_loops,
+                shared_memory_system=self.shared_memory_system,
+                output_type=self.output_type,
+                return_json=self.return_json,
+                return_entire_history=self.return_entire_history,
                 *args,
                 **kwargs,
             )
@@ -222,12 +385,14 @@ class SwarmRouter:
                 agents=self.agents,
                 max_loops=self.max_loops,
                 auto_save=self.autosave,
-                return_str_on=self.return_json,
+                return_str_on=self.return_entire_history,
                 *args,
                 **kwargs,
             )
         else:
-            raise ValueError(f"Invalid swarm type: {self.swarm_type}")
+            raise ValueError(
+                f"Invalid swarm type: {self.swarm_type} try again with a valid swarm type such as 'SequentialWorkflow' or 'ConcurrentWorkflow' or 'auto' or 'AgentRearrange' or 'MixtureOfAgents' or 'SpreadSheetSwarm'"
+            )
 
     def _log(
         self,
@@ -255,8 +420,7 @@ class SwarmRouter:
         self.logs.append(log_entry)
         logger.log(level.upper(), message)
 
-    @retry(stop=stop_after_attempt(3), wait=wait_fixed(1))
-    def run(self, task: str, *args, **kwargs) -> Any:
+    def _run(self, task: str, img: str, *args, **kwargs) -> Any:
         """
         Dynamically run the specified task on the selected or matched swarm type.
 
@@ -274,19 +438,18 @@ class SwarmRouter:
         self.swarm = self._create_swarm(task, *args, **kwargs)
 
         try:
-            self._log(
-                "info",
-                f"Running task on {self.swarm_type} swarm",
-                task=task,
-                metadata=kwargs,
-            )
-            result = self.swarm.run(task, *args, **kwargs)
-            self._log(
-                "success",
-                f"Task completed successfully on {self.swarm_type} swarm",
-                task=task,
-                metadata={"result": str(result)},
-            )
+            # self._log(
+            #     "info",
+            #     f"Running task on {self.swarm_type} swarm with task: {task}",
+            # )
+            result = self.swarm.run(task=task, *args, **kwargs)
+
+            # self._log(
+            #     "success",
+            #     f"Task completed successfully on {self.swarm_type} swarm",
+            #     task=task,
+            #     metadata={"result": str(result)},
+            # )
             return result
         except Exception as e:
             self._log(
@@ -297,6 +460,50 @@ class SwarmRouter:
             )
             raise
 
+    def run(
+        self,
+        task: str,
+        img: str = None,
+        *args,
+        **kwargs,
+    ) -> Any:
+        """
+        Execute a task on the selected swarm type with specified compute resources.
+
+        Args:
+            task (str): The task to be executed by the swarm.
+            device (str, optional): Device to run on - "cpu" or "gpu". Defaults to "cpu".
+            all_cores (bool, optional): Whether to use all CPU cores. Defaults to True.
+            all_gpus (bool, optional): Whether to use all available GPUs. Defaults to False.
+            *args: Variable length argument list.
+            **kwargs: Arbitrary keyword arguments.
+
+        Returns:
+            Any: The result of the swarm's execution.
+
+        Raises:
+            Exception: If an error occurs during task execution.
+        """
+        try:
+            return self._run(task=task, img=img, *args, **kwargs)
+        except Exception as e:
+            logger.error(f"Error executing task on swarm: {str(e)}")
+            raise
+
+    def __call__(self, task: str, *args, **kwargs) -> Any:
+        """
+        Make the SwarmRouter instance callable.
+
+        Args:
+            task (str): The task to be executed by the swarm.
+            *args: Variable length argument list.
+            **kwargs: Arbitrary keyword arguments.
+
+        Returns:
+            Any: The result of the swarm's execution.
+        """
+        return self.run(task=task, *args, **kwargs)
+
     def batch_run(
         self, tasks: List[str], *args, **kwargs
     ) -> List[Any]:
@@ -329,41 +536,6 @@ class SwarmRouter:
                 raise
         return results
 
-    def threaded_run(self, task: str, *args, **kwargs) -> Any:
-        """
-        Execute a task on the selected or matched swarm type using threading.
-
-        Args:
-            task (str): The task to be executed by the swarm.
-            *args: Variable length argument list.
-            **kwargs: Arbitrary keyword arguments.
-
-        Returns:
-            Any: The result of the swarm's execution.
-
-        Raises:
-            Exception: If an error occurs during task execution.
-        """
-        from threading import Thread
-
-        def run_in_thread():
-            try:
-                result = self.run(task, *args, **kwargs)
-                return result
-            except Exception as e:
-                self._log(
-                    "error",
-                    f"Error occurred while running task in thread on {self.swarm_type} swarm: {str(e)}",
-                    task=task,
-                    metadata={"error": str(e)},
-                )
-                raise
-
-        thread = Thread(target=run_in_thread)
-        thread.start()
-        thread.join()
-        return thread.result
-
     def async_run(self, task: str, *args, **kwargs) -> Any:
         """
         Execute a task on the selected or matched swarm type asynchronously.
@@ -424,7 +596,9 @@ class SwarmRouter:
         """
         from concurrent.futures import ThreadPoolExecutor
 
-        with ThreadPoolExecutor() as executor:
+        with ThreadPoolExecutor(
+            max_workers=os.cpu_count()
+        ) as executor:
             future = executor.submit(self.run, task, *args, **kwargs)
             result = future.result()
             return result
@@ -446,12 +620,108 @@ class SwarmRouter:
         Raises:
             Exception: If an error occurs during task execution.
         """
-        from concurrent.futures import ThreadPoolExecutor
+        from concurrent.futures import (
+            ThreadPoolExecutor,
+            as_completed,
+        )
 
+        results = []
         with ThreadPoolExecutor() as executor:
+            # Submit all tasks to executor
             futures = [
                 executor.submit(self.run, task, *args, **kwargs)
                 for task in tasks
             ]
-            results = [future.result() for future in futures]
-            return results
+
+            # Process results as they complete rather than waiting for all
+            for future in as_completed(futures):
+                try:
+                    result = future.result()
+                    results.append(result)
+                except Exception as e:
+                    logger.error(f"Task execution failed: {str(e)}")
+                    results.append(None)
+
+        return results
+
+
+def swarm_router(
+    name: str = "swarm-router",
+    description: str = "Routes your task to the desired swarm",
+    max_loops: int = 1,
+    agents: List[Union[Agent, Callable]] = [],
+    swarm_type: SwarmType = "SequentialWorkflow",  # "SpreadSheetSwarm" # "auto"
+    autosave: bool = False,
+    flow: str = None,
+    return_json: bool = True,
+    auto_generate_prompts: bool = False,
+    task: str = None,
+    rules: str = None,
+    *args,
+    **kwargs,
+) -> SwarmRouter:
+    """
+    Create and run a SwarmRouter instance with the given configuration.
+
+    Args:
+        name (str, optional): Name of the swarm router. Defaults to "swarm-router".
+        description (str, optional): Description of the router. Defaults to "Routes your task to the desired swarm".
+        max_loops (int, optional): Maximum number of execution loops. Defaults to 1.
+        agents (List[Union[Agent, Callable]], optional): List of agents or callables. Defaults to [].
+        swarm_type (SwarmType, optional): Type of swarm to use. Defaults to "SequentialWorkflow".
+        autosave (bool, optional): Whether to autosave results. Defaults to False.
+        flow (str, optional): Flow configuration. Defaults to None.
+        return_json (bool, optional): Whether to return results as JSON. Defaults to True.
+        auto_generate_prompts (bool, optional): Whether to auto-generate prompts. Defaults to False.
+        task (str, optional): Task to execute. Defaults to None.
+        *args: Additional positional arguments passed to SwarmRouter.run()
+        **kwargs: Additional keyword arguments passed to SwarmRouter.run()
+
+    Returns:
+        Any: Result from executing the swarm router
+
+    Raises:
+        ValueError: If invalid arguments are provided
+        Exception: If an error occurs during router creation or task execution
+    """
+    try:
+        logger.info(
+            f"Creating SwarmRouter with name: {name}, swarm_type: {swarm_type}"
+        )
+
+        if not agents:
+            logger.warning(
+                "No agents provided, router may have limited functionality"
+            )
+
+        if task is None:
+            logger.warning("No task provided")
+
+        swarm_router = SwarmRouter(
+            name=name,
+            description=description,
+            max_loops=max_loops,
+            agents=agents,
+            swarm_type=swarm_type,
+            autosave=autosave,
+            flow=flow,
+            return_json=return_json,
+            auto_generate_prompts=auto_generate_prompts,
+            rules=rules,
+        )
+
+        logger.info(f"Executing task with SwarmRouter: {task}")
+        result = swarm_router.run(task, *args, **kwargs)
+        logger.info(
+            f"Task execution completed successfully: {result}"
+        )
+        return result
+
+    except ValueError as e:
+        logger.error(
+            f"Invalid arguments provided to swarm_router: {str(e)}"
+        )
+        raise
+    except Exception as e:
+        logger.error(f"Error in swarm_router execution: {str(e)}")
+        raise
